      subroutine interisentrop(xt,yt,zt,thetaconst,index,itime,init,
     +firststep,iter,lkindz,dxdt,dydt)
C                              i  i  i/o    i/o      i     i    i
C         i      i     i     o    o
********************************************************************************
*                                                                              *
*     Interpolation routine for isentropic trajectories.                       *
*                                                                              *
* The user gives the vertical coordinate in metres. In the first time step of  *
* the trajectory, metres are transformed to Kelvin. In all time steps these    *
* Kelvin are kept constant and transformed onward to the eta coordinate.       *
*                                                                              *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     27 April 1994                                                            *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* dxdt,dydt          wind components in grid units per second                  *
* firststep          .true. for first step of petterssen, .false. for iteration*
* idiff [s]          Temporal distance between the windfields used for interpol*
* indz1,indz2        indices of boundary of unstable region                    *
* init               .true. for first time step of trajectory, .false. for othe*
* iter               number of iteration step                                  *
* itime [s]          current temporal position                                 *
* lkindz             unit of z coordinate (1:masl, 2:magl, 3:hPa)              *
* memtime(3) [s]     times of the wind fields in memory                        *
* nstop              =greater 0, if trajectory calculation is finished         *
* psint [Pa]         interpolated surface pressure                             *
* temp [K]           temperature at trajectory position                        *
* thetaconst [Pa]    theta level, for which trajectories shall be calculated   *
* uint,vint [m/s]    interpolated wind components                              *
* uvheight           heights in which u,v and t are given                      *
* xt,yt,zt           coordinates position for which wind data shall be calculat*
*                                                                              *
* Constants:                                                                   *
* kappa              exponent for calculating potential temperature            *
*                                                                              *
* Function:                                                                    *
* pp                 calculates the pressure at a given eta level              *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer itime,index,iter,indz1,indz2,i,lkindz
      real xt,yt,zt,dxdt,dydt,uint,vint,psint,thetaconst,phelp,pp
      real weight,uhelp,vhelp,ph1,ph2,eta
      logical init,firststep


C Calculation of interpolated surface pressure.
***********************************************

      call levlininterpol(ps,nxmax,nymax,1,nx,ny,memind,
     +xt,yt,1,memtime(index),memtime(index+1),itime,index,psint)


******************************************************************************
C For a new trajectory and the first iteration of petterssen:
C thetaconst is given in [m]
C 1. Transformation from height in [m] coordinate to height in eta coordinate.
C    Or: Transformation from hPa to eta.
C 2. Transformation from height in eta to height in theta [K] coordinate.
******************************************************************************

      if (init.and.firststep) then
        if ((lkindz.eq.1).or.(lkindz.eq.2)) then
        call etatrafo(xt,yt,thetaconst,memtime(index),memtime(index+1),
     +  itime,index,psint)  
        zt=thetaconst                  ! now thetaconst is given in eta
        else if (lkindz.eq.3) then
        zt=eta(psint,zt)
        endif

C Calculate pressure and potential temperature for current position
*******************************************************************

        phelp=pp(psint,zt)
        call potlininterpol(tt,ps,nxmax,nymax,nuvzmax,nx,ny,nuvz,
     +  memind,uvheight,akz,bkz,xt,yt,zt,memtime(index),
     +  memtime(index+1),itime,index,thetaconst)
      else

********************************************************************************
C For all other time steps:
C Transformation from height in theta [K] coordinate to height in eta coordinate
********************************************************************************

        call geteta(xt,yt,zt,thetaconst,memtime(index),memtime(index+1),
     +  itime,index,psint,indz1,indz2)
      endif


C Interpolation of wind field data is done.
*****************************************************************************

      if (indz1.eq.indz2) then         ! stable level
      if ((inpolkind.eq.1).and.(iter.ne.1)) then
        call interpol(uu,nxmax,nymax,nuvzmax,nx,ny,nuvz,memind,uvheight,
     +  xt,yt,zt,memtime(index),memtime(index+1),itime,index,uint)
        call interpol(vv,nxmax,nymax,nuvzmax,nx,ny,nuvz,memind,uvheight,
     +  xt,yt,zt,memtime(index),memtime(index+1),itime,index,vint)
      else                               ! bilinear interpolation
        call lininterpol(uu,nxmax,nymax,nuvzmax,nx,ny,nuvz,memind,
     +  uvheight,xt,yt,zt,memtime(index),memtime(index+1),itime,index,
     +  uint)
        call lininterpol(vv,nxmax,nymax,nuvzmax,nx,ny,nuvz,memind,
     +  uvheight,xt,yt,zt,memtime(index),memtime(index+1),itime,index,
     +  vint)
      endif
      else                             ! unstable layer

C -> take the layer averaged wind velocity (weighted with pressure increments)
******************************************************************************

        weight=0.
        uhelp=0.
        vhelp=0.
        ph1=akz(indz1)+bkz(indz1)*psint

        do 10 i=indz1,indz2-1
          if ((inpolkind.eq.1).and.(iter.ne.1)) then
            call levinterpol(uu,nxmax,nymax,nuvzmax,nx,ny,memind,
     +      xt,yt,i,memtime(index),memtime(index+1),itime,index,
     +      uint)
            call levinterpol(vv,nxmax,nymax,nuvzmax,nx,ny,memind,
     +      xt,yt,i,memtime(index),memtime(index+1),itime,index,
     +      vint)
          else                               ! bilinear interpolation
            call levlininterpol(uu,nxmax,nymax,nuvzmax,nx,ny,memind,
     +      xt,yt,i,memtime(index),memtime(index+1),itime,index,
     +      uint)
            call levlininterpol(vv,nxmax,nymax,nuvzmax,nx,ny,memind,
     +      xt,yt,i,memtime(index),memtime(index+1),itime,index,
     +      vint)
          endif
          ph2=akm(i+1)+bkm(i+1)*psint
          weight=weight+ph1-ph2
          uhelp=uhelp+uint*(ph1-ph2)
          vhelp=vhelp+vint*(ph1-ph2)
10        ph1=ph2

        if ((inpolkind.eq.1).and.(iter.ne.1)) then
          call levinterpol(uu,nxmax,nymax,nuvzmax,nx,ny,memind,
     +    xt,yt,indz2,memtime(index),memtime(index+1),itime,index,
     +    uint)
          call levinterpol(vv,nxmax,nymax,nuvzmax,nx,ny,memind,
     +    xt,yt,indz2,memtime(index),memtime(index+1),itime,index,
     +    vint)
        else                               ! bilinear interpolation
          call levlininterpol(uu,nxmax,nymax,nuvzmax,nx,ny,memind,
     +    xt,yt,indz2,memtime(index),memtime(index+1),itime,index,
     +    uint)
          call levlininterpol(vv,nxmax,nymax,nuvzmax,nx,ny,memind,
     +    xt,yt,indz2,memtime(index),memtime(index+1),itime,index,
     +    vint)
        endif

        ph2=akz(indz2)+bkz(indz2)*psint
        weight=weight+ph1-ph2
        uhelp=uhelp+uint*(ph1-ph2)
        vhelp=vhelp+vint*(ph1-ph2)

        uint=uhelp/weight
        vint=vhelp/weight
      endif


C TRANSFORMATION m/s --> grid units/time unit
*********************************************

      call utransform(uint,yt,dxdt)
      call vtransform(vint,dydt)


      return
      end
