      subroutine orointerpol(yy,nxmax,nymax,nx,ny,xt,yt,yint)
C                            i    i     i   i  i  i  i   o
*****************************************************************************
*                                                                           *
*  Interpolation of 3-dimensional meteorological fields.                    *
*  In horizontal direction bicubic interpolation interpolation is used.     *
*  These interpolation techniques have been found to be most accurate.      *
*                                                                           *
*  The interpolation routines have been taken from:                         *
*  Press W.H. et al. (1992): Numerical Recipes in FORTRAN. The art of       *
*  scientific computing. 2nd edition. Cambridge University Press.           *
*                                                                           *
*  But they have been modified for faster performance.                      *
*                                                                           *
*  4-3                                                                      *
*  | |  The points are numbered in this order. Values and gradients are     *
*  1-2  stored in fields with dimension 4.                                  *
*                                                                           *
*                                                                           *
*     Author: A. Stohl                                                      *
*                                                                           *
*     30 May 1994                                                           *
*                                                                           *
*****************************************************************************
*                                                                           *
* Variables:                                                                *
*                                                                           *
* ix,jy                x,y coordinates of lower left subgrid point          *
* nx,ny                actual field dimensions in x,y and z direction       *
* nxmax,nymax          maximum field dimensions in x,y and z direction      *
* x1l,x2l              x,y coordinates of lower left subgrid point          *
* x1u,x2u              x,y coordinates of upper right subgrid point         *
* xt                   current x coordinate                                 *
* y(4,1,1)             subset of 4 points for 2 times and 1 level           *
* ygx(4,1,1)           x gradients at 4 points for 2 times and 1 level      *
* ygy(4,1,1)           y gradients at 4 points for 2 times and 1 level      *
* ygxy(4,1,1)          x,y gradients at 4 points for 2 times and 1 level    *
* yhelp(1,1)           the interpolated values for 2 times and 1 level      *
* yint                 the final interpolated value                         *
* yt                   current y coordinate                                 *
* yy(0:nxmax-1,0:nymax-1) meteorological field used for interpolation       *
*                                                                           *
*****************************************************************************

      implicit none

      integer nx,ny,nxmax,nymax,i,j,l,m,n,ix,jy,im,ip,jm,jp
      real yy(0:nxmax-1,0:nymax-1)
      real y(4,1,1),ygx(4,1,1),ygy(4,1,1),ygxy(4,1,1),yhelp(1,1)
      real x1l,x1u,x2l,x2u
      real xt,yt,yint
     


C If point at border of grid -> small displacement into grid
************************************************************

      if (xt.ge.float(nx-1)) xt=float(nx-1)-0.00001
      if (yt.ge.float(ny-1)) yt=float(ny-1)-0.00001



***********************************************************************
C Bicubic horizontal interpolation
***********************************************************************

C Determine the lower left corner
********************************* 

      ix=int(xt)
      jy=int(yt)

      x1l=float(ix)
      x1u=float(ix+1)
      x2l=float(jy)
      x2u=float(jy+1)


C Loop over the 2*2 grid points
*******************************

      do 10 l=1,4
        if (l.eq.1) then
          i=ix
          j=jy
        else if (l.eq.2) then
          i=ix+1
          j=jy
        else if (l.eq.3) then
          i=ix+1
          j=jy+1
        else if (l.eq.4) then
          i=ix
          j=jy+1
        endif
        ip=i+1
        im=i-1
        jp=j+1
        jm=j-1


C Loop over 2 time steps and 3 levels
*************************************
 
        do 10 m=1,1        ! these loops and dimensions are necessary to be in 
        do 10 n=1,1        ! agreement with the full 3-d interpolation



C Values at the 2*2 subgrid
***************************

          y(l,m,n)=yy(i,j)   


C Calculate derivatives in x-direction on the 2*2 subgrid
*********************************************************

          if (i.eq.0) then
            ygx(l,m,n) = yy(ip,j )
     +                 - yy(i ,j )
          else if (i.eq.nx-1) then
            ygx(l,m,n) = yy(i ,j )
     +                 - yy(im,j )
          else
            ygx(l,m,n) =(yy(ip,j )
     +                 - yy(im,j ))/2.
          endif


C Calculate derivatives in y-direction on the 2*2 subgrid
*********************************************************

          if (j.eq.0) then
            ygy(l,m,n) = yy(i ,jp)
     +                 - yy(i ,j )
          else if (j.eq.ny-1) then
            ygy(l,m,n) = yy(i ,j )
     +                 - yy(i ,jm)
          else
            ygy(l,m,n) =(yy(i ,jp)
     +                 - yy(i ,jm))/2.
          endif


C Calculate cross derivative on the 2*2 subgrid
***********************************************

          if ((i.eq.0).and.(j.eq.0)) then
            ygxy(l,m,n)= yy(ip,jp)-
     +                   yy(ip,j )-
     +                   yy(i ,jp)+
     +                   yy(i ,j )
          else if ((i.eq.nx-1).and.(j.eq.ny-1)) then
            ygxy(l,m,n)= yy(i ,j )-
     +                   yy(i ,jm)-
     +                   yy(im,j )+
     +                   yy(im,jm)
          else if ((i.eq.0).and.(j.eq.ny-1)) then
            ygxy(l,m,n)= yy(ip,j )-
     +                   yy(ip,jm)-
     +                   yy(i ,j )+
     +                   yy(i ,jm)
          else if ((i.eq.nx-1).and.(j.eq.0)) then
            ygxy(l,m,n)= yy(i ,jp)-
     +                   yy(i ,j )-
     +                   yy(im,jp)+
     +                   yy(im,j )
          else if (i.eq.nx-1) then
            ygxy(l,m,n)=(yy(i ,jp)-
     +                   yy(i ,jm)-
     +                   yy(im,jp)+
     +                   yy(im,jm))/2.
          else if (i.eq.0) then
            ygxy(l,m,n)=(yy(ip,jp)-
     +                   yy(ip,jm)-
     +                   yy(i ,jp)+
     +                   yy(i ,jm))/2.
          else if (j.eq.ny-1) then
            ygxy(l,m,n)=(yy(ip,j )-
     +                   yy(ip,jm)-
     +                   yy(im,j )+
     +                   yy(im,jm))/2.
          else if (j.eq.0) then
            ygxy(l,m,n)=(yy(ip,jp)-
     +                   yy(ip,j )-
     +                   yy(im,jp)+
     +                   yy(im,j ))/2.
          else
            ygxy(l,m,n)=(yy(ip,jp)-
     +                   yy(ip,jm)-
     +                   yy(im,jp)+
     +                   yy(im,jm))/4.
          endif

10        continue


C Call bicubic interpolation
****************************

      call bicubic(y,ygx,ygy,ygxy,x1l,x1u,x2l,x2u,xt,yt,yhelp,1,1)

      yint=yhelp(1,1)


      return
      end
