      subroutine timemanager()                                  

********************************************************************************
*                                                                              *
* Handles the computation of trajectories, i.e. determines which               *
* trajectories have to be computed at what time.                               *
*                                                                              *
* Pointers (field numb(maxtra) are used to keep track of the trajectories.     *
* The pointers are kept sorted in the order of the next necessary time step,   *
* i.e., numb(1) points to the first trajectory that needs a time step.         *
* After doing the time step, the pointers are resorted, e.g., numb(2)          *
* changes to numb(1), numb(3) to numb(2), etc. Numb(1) is inserted between     *
* those two pointers, which need a time step before and after numb(1).         *
*                                                                              *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     6 February 1994                                                          *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* ideltas [s]        modelling period                                          *
* init               .true. for first time step of trajectory, false .else.    *
* interv [s]         interval between two trajectory starting times            *
* ischeme            kind of integration scheme for trajectories:              *
*                    1 = Petterssen, 2 = Pudykiewicz                           *
* itime [s]          actual temporal position of calculation                   *
* ittra(maxtra,maxtime) temporal positions of trajectories                     *
* kind(maxtra)       type of trajectory (e.g. isobaric, 3-d, model layers,..)  *
* kindz(maxtra)      unit of height coordinate (1:masl, 2:magl, 3:hPa)         *
* ldirect            Temporal direction of trajectories (-1=backward,1=forward)*
* lentra             temporal length of one trajectory                         *
* levmem(maxtra) [K] height of isentropic trajectories                         *
* minstep            minimum time step (to prevent exceedance of dimensions)   *
* npoint(maxtra)     index, which starting point the trajectory has            *
* nstop              = greater than 0, if trajectory calculation is finished   *
* ntstep [s]         time step of integration scheme                           *
* nttra(maxtra)      number of time steps which are already computed           *
* numb(maxtra)       pointer, which allows the identification of trajectories  *
* number             help variable                                             *
* numtra             actual number of trajectories in memory                   *
* xpoint(maxpoint), ypoint(maxpoint),zpoint(maxpoint) =                        *
* xpoint(maxpoint), ypoint(maxpoint),zpoint(maxpoint) =                        *
*                    starting positions of trajectories                        *
* xtra(maxtra,maxtime), ytra(maxtra,maxtime), ztra(maxtra,maxtime) =           *
*                    spatial positions of trajectories                         *
*                                                                              *
* Constants:                                                                   *
* maxtra             maximum number of trajectories                            *
* maxtime            maximum number of time steps                              *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer i,j,itime,ntstep,nstop(maxtra),lhelp,minstep
      integer numb(maxtra),number
      real levmem(maxtra)
      logical init,unc


      do 5 i=1,maxtra
5       nttra(i)=0
      numtra=0                                      ! first: no trajectories


C Loop over the whole modelling period in time steps of seconds
***************************************************************

      do 10 itime=0,ideltas,ldirect


C Check, if some trajectories have already finished. If so, call output
C and make memory available for new trajectory.
***********************************************************************

        if (numtra.gt.0) then

          i=1
33        continue
          number=numb(i)
       
          if (abs(ittra(number,nttra(number))).gt.abs(itime)) goto 37
          if ((nstop(number).gt.0).and.               ! trajectory terminated
     +       (ittra(number,nttra(number)).eq.itime)) then     ! output time
            call trajout(number,nstop(number))        ! output of trajectory
            nttra(number)=0                           ! reinitialize
            do 35 j=i,numtra-1                        ! resort the pointers
35            numb(j)=numb(j+1)
            numtra=numtra-1                           ! forget one trajectory
            i=i-1
          endif
          if (i.lt.numtra) then
            i=i+1
            goto 33
          else
            goto 37 
          endif

37        continue
        endif



C Check, if new trajectories have to be started
***********************************************

        if (abs(itime).le.abs(ideltas-lentra)) then ! if time within period
        if (mod(itime,interv).eq.0) then            ! if interv has passed by
          do 40 i=numtra,1,-1                       ! shift pointers
40          numb(i+numpoint)=numb(i)
          numtra=numtra+numpoint                    ! increase number of traj.


C Initialize new trajectories
*****************************

          j=1
          do 50 i=1,numpoint                        !search for available memory
55          continue
            if (nttra(j).eq.0) then
              numb(i)=j
              npoint(j)=i
              levmem(j)=zpoint(i)
              xtra(j,1)=xpoint(i)
              ytra(j,1)=ypoint(i)
              ztra(j,1)=zpoint(i)
              ittra(j,1)=itime
              nttra(j)=1
              kind(j)=kind(i)
              kindz(j)=kindz(i)
            else 
              j=j+1
              goto 55
            endif
50          continue


        endif
        endif



C Check, if a time step of the first trajectory is necessary
************************************************************

15      continue
        if (numtra.gt.0) then
        
        number=numb(1)

        if (ittra(number,nttra(number)).eq.itime) then

C minstep is the minimum possible time step. It is necessary to 
C guarantee that the sum of all time steps along the trajectory
C is smaller than maxtime (otherwise exceedance of dimensions).
***************************************************************

          lhelp=itime-ittra(number,1)         ! "age" of trajectory
          minstep=abs(lentra-lhelp)/(maxtime-nttra(number))+1


C Mark first time step of trajectory with variable init=.TRUE.
**************************************************************

          if (nttra(number).eq.1) then
            init=.true.
          else
            init=.false.
          endif


C Determine, if current trajectory is an uncertainty trajectory
***************************************************************

          if (compoint(npoint(number))(41:41).eq.'U') then
            unc=.TRUE.
          else
            unc=.FALSE.
          endif
           

C Calculate the next step of the trajectory
*******************************************

          if (ischeme.eq.1) then                ! use Petterssen scheme

          call petters(minstep,lhelp,itime,xtra(number,nttra(number)),
     +    ytra(number,nttra(number)),ztra(number,nttra(number)),
     +    ptra(number,nttra(number)),htra(number,nttra(number)),
     +    vorttra(number,nttra(number)),
     +    xtra(number,nttra(number)+1),ytra(number,nttra(number)+1),
     +    ztra(number,nttra(number)+1),ptra(number,nttra(number)+1),
     +    htra(number,nttra(number)+1),vorttra(number,nttra(number)+1),
     +    ntstep,nstop(number),levmem(number),init,kind(number),
     +    kindz(number),unc,pottem(number,nttra(number)),
     +    pottem(number,nttra(number)+1))

          else if (ischeme.eq.2) then           ! use Pudykiewicz scheme

          call pudykiew(minstep,lhelp,itime,xtra(number,nttra(number))
     +    ,ytra(number,nttra(number)),ztra(number,nttra(number)),
     +    ptra(number,nttra(number)),htra(number,nttra(number)),
     +    vorttra(number,nttra(number)),
     +    xtra(number,nttra(number)+1),ytra(number,nttra(number)+1),
     +    ztra(number,nttra(number)+1),ptra(number,nttra(number)+1),
     +    htra(number,nttra(number)+1),vorttra(number,nttra(number)+1),
     +    ntstep,nstop(number),levmem(number),init,kind(number),
     +    kindz(number),unc,pottem(number,nttra(number)),
     +    pottem(number,nttra(number)+1))

          endif

          ittra(number,nttra(number)+1)=ittra(number,nttra(number))+
     +    ntstep

          nttra(number)=nttra(number)+1         ! one more time step

C If field dimension is reached, trajectory has to be terminated
****************************************************************

          if (nttra(number).eq.maxtime) nstop(number)=5


C If trajectory has ended, give the next time step the time of normal
C length of trajectory and wait for output of trajectory
*********************************************************************

          if (nstop(number).gt.0) then
            ittra(number,nttra(number))=ittra(number,1)+lentra
          endif


C Shift the trajectories in a way that the current trajectory can be
C inserted at the right position.
C The position is determined by the time, when the next integration step
C is necessary.
*************************************************************************

          do 20 i=1,numtra-1
            if (abs(ittra(numb(i+1),nttra(numb(i+1)))).lt.
     +      abs(ittra(number,nttra(number)))) then
              numb(i)=numb(i+1)
            else
              numb(i)=number                      ! insert the pointer
              goto 15                             ! leave loop
            endif
20          continue

C The next statement can only be reached, when the current trajectory is the
C last one that needs an integration step.
****************************************************************************

          numb(numtra)=number
          goto 15                                 ! check next trajectory
        endif
        endif


10     continue

      return
      end
